#ifndef PERL_PROTOBUF_T_C_UTILS_H_
#define PERL_PROTOBUF_T_C_UTILS_H_

#include "EXTERN.h"
#include "perl.h"  // NOLINT(build/include)

void run_utils_tests(pTHX);

#endif  // PERL_PROTOBUF_T_C_UTILS_H_
