#ifndef PERL_PROTOBUF_CONVERT_TYPES_MESSAGE_H_
#define PERL_PROTOBUF_CONVERT_TYPES_MESSAGE_H_

#include "t/c/convert/test_util.h"

// Declare functions and data related to message tests
extern const upb_to_sv_test_case message_upb_to_sv_test_cases[];
extern const sv_to_upb_test_case message_sv_to_upb_test_cases[];

#endif  // PERL_PROTOBUF_CONVERT_TYPES_MESSAGE_H_
