// LINT: LEGACY_NAMES
syntax = "proto2";

package Local.Messages;

option optimize_for = SPEED;

message One {
  required int32 id1 = 1;
}

message Two {
  required int32 id2 = 1;
}

message Three {
  required int32 id3 = 1;
}
